/*
 * Copyright (C) 2010 David Arkell.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.gwtorbit.client.widgets;

import java.util.ArrayList;

import com.allen_sauer.gwt.dnd.client.PickupDragController;
import com.allen_sauer.gwt.dnd.client.drop.VerticalPanelDropController;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.gwtorbit.client.model.DisplayMethods;
import com.gwtorbit.client.model.UiData;
import com.gwtorbit.client.model.UiDataDisplay;
import com.gwtorbit.client.model.UiErrorDisplay;
import com.gwtorbit.client.model.UiErrors;
import com.gwtorbit.client.model.UiLocalizedError;

public abstract class UiDragAndDropVerticalList extends AbsolutePanel implements UiDataDisplay,
		UiErrorDisplay {

	protected abstract Widget createDisplayInstance(UiData uiData);

	protected abstract Widget getDragHandle(Widget w);

	protected abstract UiData createDataInstance(Widget w);

	private PickupDragController widgetDragController;

	private VerticalPanel verticalPanel;

	private VerticalPanelDropController widgetDropController;

	private boolean initialised = false;

	public void init() {
		if (!initialised) {
			initialised = true;

			verticalPanel = new VerticalPanel();
			add(verticalPanel, 0, 0);

			widgetDragController = new PickupDragController(this, false);
			widgetDragController.setBehaviorMultipleSelection(false);
			widgetDragController.setConstrainWidgetToBoundaryPanel(true);
			widgetDragController.setBehaviorBoundaryPanelDrop(false);
			widgetDragController.setBehaviorConstrainedToBoundaryPanel(true);

			// initialize a widget drop controller for the current column
			widgetDropController = new VerticalPanelDropController(verticalPanel);
			widgetDragController.registerDropController(widgetDropController);
		}
	}

	private void recalculateHeight() {
		int totalHeight = 0;
		for (int i = 0; i < verticalPanel.getWidgetCount(); i++) {
			totalHeight += verticalPanel.getWidget(i).getOffsetHeight();
		}
		setHeight(totalHeight + "px");
	}

	@SuppressWarnings("unchecked")
	public void displayData(UiData uiData) {
		ArrayList<UiData> dataList = (ArrayList<UiData>) uiData.getData(dataName);

		clearData();
		for (UiData tmpData : dataList) {
			addItem(tmpData);
		}
	}

	public void getData(UiData uiData, UiErrors errors, String errorPrefix) {
		ArrayList<UiData> dataList = new ArrayList<UiData>();

		for (int i = 0; i < verticalPanel.getWidgetCount(); i++) {
			Widget tmpDisplay = verticalPanel.getWidget(i);
			UiData tmpData = createDataInstance(tmpDisplay);

			DisplayMethods.getData(tmpDisplay, tmpData, errors, errorPrefix + dataName + "[" + i
					+ "].");
			dataList.add(tmpData);
		}

		if (mandatory && verticalPanel.getWidgetCount() == 0) {
			errors.addError(errorPrefix + dataName, UiLocalizedError.MANDATORY);
		}

		uiData.setData(dataName, dataList);
	}

	private String dataName;

	public String getDataName() {
		return dataName;
	}

	public void setDataName(String dataName) {
		this.dataName = dataName;
	}

	public void clearErrors() {
		init();

		for (int i = 0; i < verticalPanel.getWidgetCount(); i++) {
			Widget tmpDisplay = verticalPanel.getWidget(i);
			DisplayMethods.clearErrors(tmpDisplay);
		}
	}

	public void displayErrors(UiErrors errors, String errorPrefix) {
		for (int i = 0; i < verticalPanel.getWidgetCount(); i++) {
			Widget tmpDisplay = verticalPanel.getWidget(i);
			DisplayMethods.displayErrors(tmpDisplay, errors, errorPrefix + dataName + "[" + i
					+ "].");
		}
	}

	public void clearData() {
		if (verticalPanel != null) {
			verticalPanel.clear();
		}
	}

	public void addItem(UiData uiData) {
		addItem(createDisplayInstance(uiData), uiData);
	}

	public void addItem(Widget widget, UiData uiData) {
		init();
		verticalPanel.add(widget);
		DisplayMethods.displayData(widget, uiData);
		recalculateHeight();
		widgetDragController.makeDraggable(widget, getDragHandle(widget));
	}

	public void removeItem(Widget widget) {
		verticalPanel.remove(widget);
		recalculateHeight();
	}

	private boolean ignore = false;

	public boolean getIgnore() {
		return ignore;
	}

	public void setIgnore(boolean ignore) {
		this.ignore = ignore;
	}

	public void setMandatory(boolean mandatory) {
		this.mandatory = mandatory;
	}

	public boolean isMandatory() {
		return mandatory;
	}

	private boolean mandatory = false;

}
